@@ -5,7 +5,7 @@ Matrix equation solvers (Lyapunov, Riccati)
 Implementation of the functions lyap, dlyap, care and dare
 for solution of Lyapunov and Riccati equations. """
 
-# Python 3 compatability (needs to go here)
+# Python 3 compatibility (needs to go here)
 from __future__ import print_function
 
 """Copyright (c) 2011, All rights reserved.
@@ -41,31 +41,32 @@ SUCH DAMAGE.
 Author: Bjorn Olofsson
 """
 
-from numpy.linalg import inv
 from scipy import shape, size, asarray, asmatrix, copy, zeros, eye, dot
-from scipy.linalg import eigvals, solve_discrete_are
+from scipy.linalg import eigvals, solve_discrete_are, solve
 from .exception import ControlSlycot, ControlArgument
 
+__all__ = ['lyap', 'dlyap', 'dare', 'care']
+
 #### Lyapunov equation solvers lyap and dlyap
 
 def lyap(A,Q,C=None,E=None):
     """ X = lyap(A,Q) solves the continuous-time Lyapunov equation
 
-        A X + X A^T + Q = 0
+        :math:`A X + X A^T + Q = 0`
 
     where A and Q are square matrices of the same dimension.
     Further, Q must be symmetric.
 
     X = lyap(A,Q,C) solves the Sylvester equation
 
-        A X + X Q + C = 0
+        :math:`A X + X Q + C = 0`
 
     where A and Q are square matrices.
 
     X = lyap(A,Q,None,E) solves the generalized continuous-time
     Lyapunov equation
 
-        A X E^T + E X A^T + Q = 0
+        :math:`A X E^T + E X A^T + Q = 0`
 
     where Q is a symmetric matrix and A, Q and E are square matrices
     of the same dimension. """
@@ -88,10 +89,10 @@ def lyap(A,Q,C=None,E=None):
     if len(shape(Q)) == 1:
         Q = Q.reshape(1,Q.size)
 
-    if C != None and len(shape(C)) == 1:
+    if C is not None and len(shape(C)) == 1:
         C = C.reshape(1,C.size)
 
-    if E != None and len(shape(E)) == 1:
+    if E is not None and len(shape(E)) == 1:
         E = E.reshape(1,E.size)
 
     # Determine main dimensions
@@ -106,7 +107,7 @@ def lyap(A,Q,C=None,E=None):
         m = size(Q,0)
 
     # Solve standard Lyapunov equation
-    if C==None and E==None:
+    if C is None and E is None:
         # Check input data for consistency
         if shape(A) != shape(Q):
             raise ControlArgument("A and Q must be matrices of identical \
@@ -139,7 +140,7 @@ def lyap(A,Q,C=None,E=None):
             raise e
 
     # Solve the Sylvester equation
-    elif C != None and E==None:
+    elif C is not None and E is None:
         # Check input data for consistency
         if size(A) > 1 and shape(A)[0] != shape(A)[1]:
             raise ControlArgument("A must be a quadratic matrix.")
@@ -170,7 +171,7 @@ def lyap(A,Q,C=None,E=None):
             raise e
 
     # Solve the generalized Lyapunov equation
-    elif C == None and E != None:
+    elif C is None and E is not None:
         # Check input data for consistency
         if (size(Q) > 1 and shape(Q)[0] != shape(Q)[1]) or \
             (size(Q) > 1 and shape(Q)[0] != n) or \
@@ -233,21 +234,21 @@ def lyap(A,Q,C=None,E=None):
 def dlyap(A,Q,C=None,E=None):
     """ dlyap(A,Q) solves the discrete-time Lyapunov equation
 
-        A X A^T - X + Q = 0
+        :math:`A X A^T - X + Q = 0`
 
     where A and Q are square matrices of the same dimension. Further
     Q must be symmetric.
 
     dlyap(A,Q,C) solves the Sylvester equation
 
-        A X Q^T - X + C = 0
+        :math:`A X Q^T - X + C = 0`
 
     where A and Q are square matrices.
 
     dlyap(A,Q,None,E) solves the generalized discrete-time Lyapunov
     equation
 
-        A X A^T - E X E^T + Q = 0
+        :math:`A X A^T - E X E^T + Q = 0`
 
     where Q is a symmetric matrix and A, Q and E are square matrices
     of the same dimension. """
@@ -275,10 +276,10 @@ def dlyap(A,Q,C=None,E=None):
     if len(shape(Q)) == 1:
         Q = Q.reshape(1,Q.size)
 
-    if C != None and len(shape(C)) == 1:
+    if C is not None and len(shape(C)) == 1:
         C = C.reshape(1,C.size)
 
-    if E != None and len(shape(E)) == 1:
+    if E is not None and len(shape(E)) == 1:
         E = E.reshape(1,E.size)
 
     # Determine main dimensions
@@ -293,7 +294,7 @@ def dlyap(A,Q,C=None,E=None):
         m = size(Q,0)
 
     # Solve standard Lyapunov equation
-    if C==None and E==None:
+    if C is None and E is None:
         # Check input data for consistency
         if shape(A) != shape(Q):
             raise ControlArgument("A and Q must be matrices of identical \
@@ -322,7 +323,7 @@ def dlyap(A,Q,C=None,E=None):
             raise e
 
     # Solve the Sylvester equation
-    elif C != None and E==None:
+    elif C is not None and E is None:
         # Check input data for consistency
         if size(A) > 1 and shape(A)[0] != shape(A)[1]:
             raise ControlArgument("A must be a quadratic matrix")
@@ -353,7 +354,7 @@ def dlyap(A,Q,C=None,E=None):
             raise e
 
     # Solve the generalized Lyapunov equation
-    elif C == None and E != None:
+    elif C is None and E is not None:
         # Check input data for consistency
         if (size(Q) > 1 and shape(Q)[0] != shape(Q)[1]) or \
             (size(Q) > 1 and shape(Q)[0] != n) or \
@@ -411,25 +412,27 @@ def dlyap(A,Q,C=None,E=None):
 #### Riccati equation solvers care and dare
 
 def care(A,B,Q,R=None,S=None,E=None):
-    """ (X,L,G) = care(A,B,Q) solves the continuous-time algebraic Riccati
+    """ (X,L,G) = care(A,B,Q,R=None) solves the continuous-time algebraic Riccati
     equation
 
-        A^T X + X A - X B B^T X + Q = 0
+        :math:`A^T X + X A - X B R^{-1} B^T X + Q = 0`
 
-    where A and Q are square matrices of the same dimension. Further, Q
-    is a symmetric matrix. The function returns the solution X, the gain
-    matrix G = B^T X and the closed loop eigenvalues L, i.e., the eigenvalues
-    of A - B G.
+    where A and Q are square matrices of the same dimension. Further,
+    Q and R are a symmetric matrices. If R is None, it is set to the
+    identity matrix. The function returns the solution X, the gain
+    matrix G = B^T X and the closed loop eigenvalues L, i.e., the
+    eigenvalues of A - B G.
 
     (X,L,G) = care(A,B,Q,R,S,E) solves the generalized continuous-time
     algebraic Riccati equation
 
-        A^T X E + E^T X A - (E^T X B + S) R^-1 (B^T X E + S^T) + Q = 0
+        :math:`A^T X E + E^T X A - (E^T X B + S) R^{-1} (B^T X E + S^T) + Q = 0`
 
-    where A, Q and E are square matrices of the same dimension. Further, Q and
-    R are symmetric matrices. The function returns the solution X, the gain
-    matrix G = R^-1 (B^T X E + S^T) and the closed loop eigenvalues L, i.e.,
-    the eigenvalues of A - B G , E. """
+    where A, Q and E are square matrices of the same
+    dimension. Further, Q and R are symmetric matrices. If R is None,
+    it is set to the identity matrix. The function returns the
+    solution X, the gain matrix G = R^-1 (B^T X E + S^T) and the
+    closed loop eigenvalues L, i.e., the eigenvalues of A - B G , E."""
 
     # Make sure we can import required slycot routine
     try:
@@ -458,13 +461,13 @@ def care(A,B,Q,R=None,S=None,E=None):
     if len(shape(Q)) == 1:
         Q = Q.reshape(1,Q.size)
 
-    if R != None and len(shape(R)) == 1:
+    if R is not None and len(shape(R)) == 1:
         R = R.reshape(1,R.size)
 
-    if S != None and len(shape(S)) == 1:
+    if S is not None and len(shape(S)) == 1:
         S = S.reshape(1,S.size)
 
-    if E != None and len(shape(E)) == 1:
+    if E is not None and len(shape(E)) == 1:
         E = E.reshape(1,E.size)
 
     # Determine main dimensions
@@ -477,11 +480,11 @@ def care(A,B,Q,R=None,S=None,E=None):
         m = 1
     else:
         m = size(B,1)
-    if R==None:
+    if R is None:
         R = eye(m,m)
 
     # Solve the standard algebraic Riccati equation
-    if S==None and E==None:
+    if S is None and E is None:
         # Check input data for consistency
         if size(A) > 1 and shape(A)[0] != shape(A)[1]:
             raise ControlArgument("A must be a quadratic matrix.")
@@ -531,7 +534,7 @@ def care(A,B,Q,R=None,S=None,E=None):
                 e.info = ve.info
             elif ve.info == 1:
                 e = ValueError("The matrix A is (numerically) singular in \
-                    discrete-time case.")
+                    continuous-time case.")
                 e.info = ve.info
             elif ve.info == 2:
                 e = ValueError("The Hamiltonian or symplectic matrix H cannot \
@@ -553,16 +556,16 @@ def care(A,B,Q,R=None,S=None,E=None):
 
         # Calculate the gain matrix G
         if size(R_b) == 1:
-            G = dot(dot(1/(R_ba),asarray(B_ba).T) , X)
+            G = dot(dot(1/(R_ba), asarray(B_ba).T), X)
         else:
-            G = dot(dot(inv(R_ba),asarray(B_ba).T) , X)
+            G = dot(solve(R_ba, asarray(B_ba).T), X)
 
         # Return the solution X, the closed-loop eigenvalues L and
         # the gain matrix G
         return (X , w[:n] , G )
 
     # Solve the generalized algebraic Riccati equation
-    elif S != None and E != None:
+    elif S is not None and E is not None:
         # Check input data for consistency
         if size(A) > 1 and shape(A)[0] != shape(A)[1]:
             raise ControlArgument("A must be a quadratic matrix.")
@@ -656,9 +659,9 @@ def care(A,B,Q,R=None,S=None,E=None):
 
         # Calculate the gain matrix G
         if size(R_b) == 1:
-            G = dot(1/(R_b),dot(asarray(B_b).T,dot(X,E_b))+asarray(S_b).T)
+            G = dot(1/(R_b), dot(asarray(B_b).T, dot(X,E_b)) + asarray(S_b).T)
         else:
-            G = dot(inv(R_b),dot(asarray(B_b).T,dot(X,E_b))+asarray(S_b).T)
+            G = solve(R_b, dot(asarray(B_b).T, dot(X, E_b)) + asarray(S_b).T)
 
         # Return the solution X, the closed-loop eigenvalues L and
         # the gain matrix G
@@ -672,7 +675,7 @@ def dare(A,B,Q,R,S=None,E=None):
     """ (X,L,G) = dare(A,B,Q,R) solves the discrete-time algebraic Riccati
     equation
 
-        A^T X A - X - A^T X B (B^T X B + R)^-1 B^T X A + Q = 0
+        :math:`A^T X A - X - A^T X B (B^T X B + R)^{-1} B^T X A + Q = 0`
 
     where A and Q are square matrices of the same dimension. Further, Q
     is a symmetric matrix. The function returns the solution X, the gain
@@ -682,12 +685,11 @@ def dare(A,B,Q,R,S=None,E=None):
     (X,L,G) = dare(A,B,Q,R,S,E) solves the generalized discrete-time algebraic
     Riccati equation
 
-        A^T X A - E^T X E - (A^T X B + S) (B^T X B + R)^-1 (B^T X A + S^T) +
-            + Q = 0
+        :math:`A^T X A - E^T X E - (A^T X B + S) (B^T X B + R)^{-1} (B^T X A + S^T) + Q = 0`
 
     where A, Q and E are square matrices of the same dimension. Further, Q and
     R are symmetric matrices. The function returns the solution X, the gain
-    matrix G = (B^T X B + R)^-1 (B^T X A + S^T) and the closed loop
+    matrix :math:`G = (B^T X B + R)^{-1} (B^T X A + S^T)` and the closed loop
     eigenvalues L, i.e., the eigenvalues of A - B G , E.
     """
     if S is not None or E is not None:
@@ -696,7 +698,7 @@ def dare(A,B,Q,R,S=None,E=None):
         Rmat = asmatrix(R)
         Qmat = asmatrix(Q)
         X = solve_discrete_are(A, B, Qmat, Rmat)
-        G = inv(B.T.dot(X).dot(B) + Rmat) * B.T.dot(X).dot(A)
+        G = solve(B.T.dot(X).dot(B) + Rmat, B.T.dot(X).dot(A))
         L = eigvals(A - B.dot(G))
         return X, L, G
 
@@ -728,13 +730,13 @@ def dare_old(A,B,Q,R,S=None,E=None):
     if len(shape(Q)) == 1:
         Q = Q.reshape(1,Q.size)
 
-    if R != None and len(shape(R)) == 1:
+    if R is not None and len(shape(R)) == 1:
         R = R.reshape(1,R.size)
 
-    if S != None and len(shape(S)) == 1:
+    if S is not None and len(shape(S)) == 1:
         S = S.reshape(1,S.size)
 
-    if E != None and len(shape(E)) == 1:
+    if E is not None and len(shape(E)) == 1:
         E = E.reshape(1,E.size)
 
     # Determine main dimensions
@@ -749,7 +751,7 @@ def dare_old(A,B,Q,R,S=None,E=None):
         m = size(B,1)
 
     # Solve the standard algebraic Riccati equation
-    if S==None and E==None:
+    if S is None and E is None:
         # Check input data for consistency
         if size(A) > 1 and shape(A)[0] != shape(A)[1]:
             raise ControlArgument("A must be a quadratic matrix.")
@@ -822,18 +824,18 @@ def dare_old(A,B,Q,R,S=None,E=None):
 
         # Calculate the gain matrix G
         if size(R_b) == 1:
-            G = dot( 1/(dot(asarray(B_ba).T,dot(X,B_ba))+R_ba) , \
-                dot(asarray(B_ba).T,dot(X,A_ba)) )
+            G = dot(1/(dot(asarray(B_ba).T, dot(X, B_ba)) + R_ba), \
+                dot(asarray(B_ba).T, dot(X, A_ba)))
         else:
-            G = dot( inv(dot(asarray(B_ba).T,dot(X,B_ba))+R_ba) , \
-                dot(asarray(B_ba).T,dot(X,A_ba)) )
+            G = solve(dot(asarray(B_ba).T, dot(X, B_ba)) + R_ba, \
+                dot(asarray(B_ba).T, dot(X, A_ba)))
 
         # Return the solution X, the closed-loop eigenvalues L and
         # the gain matrix G
         return (X , w[:n] , G)
 
     # Solve the generalized algebraic Riccati equation
-    elif S != None and E != None:
+    elif S is not None and E is not None:
         # Check input data for consistency
         if size(A) > 1 and shape(A)[0] != shape(A)[1]:
             raise ControlArgument("A must be a quadratic matrix.")
@@ -927,11 +929,11 @@ def dare_old(A,B,Q,R,S=None,E=None):
 
         # Calculate the gain matrix G
         if size(R_b) == 1:
-            G = dot( 1/(dot(asarray(B_b).T,dot(X,B_b))+R_b) , \
-                dot(asarray(B_b).T,dot(X,A_b)) + asarray(S_b).T)
+            G = dot(1/(dot(asarray(B_b).T, dot(X,B_b)) + R_b), \
+                dot(asarray(B_b).T, dot(X,A_b)) + asarray(S_b).T)
         else:
-            G = dot( inv(dot(asarray(B_b).T,dot(X,B_b))+R_b) , \
-                dot(asarray(B_b).T,dot(X,A_b)) + asarray(S_b).T)
+            G = solve(dot(asarray(B_b).T, dot(X,B_b)) + R_b, \
+                dot(asarray(B_b).T, dot(X,A_b)) + asarray(S_b).T)
 
         # Return the solution X, the closed-loop eigenvalues L and
         # the gain matrix G
