@@ -15,6 +15,8 @@ from scipy.linalg import eigvals
 import scipy as sp
 from control.matlab import *
 from control.frdata import FRD
+from control.exception import slycot_check
+import warnings
 
 # for running these through Matlab or Octave
 '''
@@ -133,11 +135,19 @@ class TestMatlab(unittest.TestCase):
         pzmap(self.siso_tf2, Plot=False);
 
     def testStep(self):
-        #Test SISO system
-        sys = self.siso_ss1
         t = np.linspace(0, 1, 10)
+        # Test transfer function
+        yout, tout = step(self.siso_tf1, T=t)
+        youttrue = np.array([0, 0.0057, 0.0213, 0.0446, 0.0739,
+                             0.1075, 0.1443, 0.1832, 0.2235, 0.2642])
+        np.testing.assert_array_almost_equal(yout, youttrue, decimal=4)
+        np.testing.assert_array_almost_equal(tout, t)
+
+        # Test SISO system with direct feedthrough
+        sys = self.siso_ss1
         youttrue = np.array([9., 17.6457, 24.7072, 30.4855, 35.2234, 39.1165,
                              42.3227, 44.9694, 47.1599, 48.9776])
+
         yout, tout = step(sys, T=t)
         np.testing.assert_array_almost_equal(yout, youttrue, decimal=4)
         np.testing.assert_array_almost_equal(tout, t)
@@ -152,29 +162,59 @@ class TestMatlab(unittest.TestCase):
         np.testing.assert_array_almost_equal(yout, youttrue, decimal=4)
         np.testing.assert_array_almost_equal(tout, t)
 
-        #Test MIMO system, which contains ``siso_ss1`` twice
-        sys = self.mimo_ss1
-        y_00, _t = step(sys, T=t, input=0, output=0)
-        y_11, _t = step(sys, T=t, input=1, output=1)
-        np.testing.assert_array_almost_equal(y_00, youttrue, decimal=4)
-        np.testing.assert_array_almost_equal(y_11, youttrue, decimal=4)
-
-    def testImpulse(self):
-        #Test SISO system
-        sys = self.siso_ss1
-        t = np.linspace(0, 1, 10)
-        youttrue = np.array([86., 70.1808, 57.3753, 46.9975, 38.5766, 31.7344,
-                             26.1668, 21.6292, 17.9245, 14.8945])
-        yout, tout = impulse(sys, T=t)
+        yout, tout, xout = step(sys, T=t, X0=0, return_x=True)
         np.testing.assert_array_almost_equal(yout, youttrue, decimal=4)
         np.testing.assert_array_almost_equal(tout, t)
 
-        #Test MIMO system, which contains ``siso_ss1`` twice
-        sys = self.mimo_ss1
-        y_00, _t = impulse(sys, T=t, input=0, output=0)
-        y_11, _t = impulse(sys, T=t, input=1, output=1)
-        np.testing.assert_array_almost_equal(y_00, youttrue, decimal=4)
-        np.testing.assert_array_almost_equal(y_11, youttrue, decimal=4)
+        if slycot_check():
+            # Test MIMO system, which contains ``siso_ss1`` twice
+            sys = self.mimo_ss1
+            y_00, _t = step(sys, T=t, input=0, output=0)
+            y_11, _t = step(sys, T=t, input=1, output=1)
+            np.testing.assert_array_almost_equal(y_00, youttrue, decimal=4)
+            np.testing.assert_array_almost_equal(y_11, youttrue, decimal=4)
+
+    def testImpulse(self):
+        t = np.linspace(0, 1, 10)
+        # test transfer function
+        yout, tout = impulse(self.siso_tf1, T=t)
+        youttrue = np.array([0., 0.0994, 0.1779, 0.2388, 0.2850, 0.3188,
+                             0.3423, 0.3573, 0.3654, 0.3679])
+        np.testing.assert_array_almost_equal(yout, youttrue, decimal=4)
+        np.testing.assert_array_almost_equal(tout, t)
+
+        # produce a warning for a system with direct feedthrough
+        with warnings.catch_warnings():
+            warnings.simplefilter("ignore")
+            #Test SISO system
+            sys = self.siso_ss1
+            youttrue = np.array([86., 70.1808, 57.3753, 46.9975, 38.5766, 31.7344,
+                                 26.1668, 21.6292, 17.9245, 14.8945])
+            yout, tout = impulse(sys, T=t)
+            np.testing.assert_array_almost_equal(yout, youttrue, decimal=4)
+            np.testing.assert_array_almost_equal(tout, t)
+
+            # Play with arguments
+            yout, tout = impulse(sys, T=t, X0=0)
+            np.testing.assert_array_almost_equal(yout, youttrue, decimal=4)
+            np.testing.assert_array_almost_equal(tout, t)
+
+            X0 = np.array([0, 0]);
+            yout, tout = impulse(sys, T=t, X0=X0)
+            np.testing.assert_array_almost_equal(yout, youttrue, decimal=4)
+            np.testing.assert_array_almost_equal(tout, t)
+
+            yout, tout, xout = impulse(sys, T=t, X0=0, return_x=True)
+            np.testing.assert_array_almost_equal(yout, youttrue, decimal=4)
+            np.testing.assert_array_almost_equal(tout, t)
+
+            if slycot_check():
+                #Test MIMO system, which contains ``siso_ss1`` twice
+                sys = self.mimo_ss1
+                y_00, _t = impulse(sys, T=t, input=0, output=0)
+                y_11, _t = impulse(sys, T=t, input=1, output=1)
+                np.testing.assert_array_almost_equal(y_00, youttrue, decimal=4)
+                np.testing.assert_array_almost_equal(y_11, youttrue, decimal=4)
 
     def testInitial(self):
         #Test SISO system
@@ -187,13 +227,19 @@ class TestMatlab(unittest.TestCase):
         np.testing.assert_array_almost_equal(yout, youttrue, decimal=4)
         np.testing.assert_array_almost_equal(tout, t)
 
-        #Test MIMO system, which contains ``siso_ss1`` twice
-        sys = self.mimo_ss1
-        x0 = np.matrix(".5; 1.; .5; 1.")
-        y_00, _t = initial(sys, T=t, X0=x0, input=0, output=0)
-        y_11, _t = initial(sys, T=t, X0=x0, input=1, output=1)
-        np.testing.assert_array_almost_equal(y_00, youttrue, decimal=4)
-        np.testing.assert_array_almost_equal(y_11, youttrue, decimal=4)
+        # Play with arguments
+        yout, tout, xout = initial(sys, T=t, X0=x0, return_x=True)
+        np.testing.assert_array_almost_equal(yout, youttrue, decimal=4)
+        np.testing.assert_array_almost_equal(tout, t)
+
+        if slycot_check():
+            #Test MIMO system, which contains ``siso_ss1`` twice
+            sys = self.mimo_ss1
+            x0 = np.matrix(".5; 1.; .5; 1.")
+            y_00, _t = initial(sys, T=t, X0=x0, input=0, output=0)
+            y_11, _t = initial(sys, T=t, X0=x0, input=1, output=1)
+            np.testing.assert_array_almost_equal(y_00, youttrue, decimal=4)
+            np.testing.assert_array_almost_equal(y_11, youttrue, decimal=4)
 
     def testLsim(self):
         t = np.linspace(0, 1, 10)
@@ -217,18 +263,19 @@ class TestMatlab(unittest.TestCase):
         yout, _t, _xout = lsim(self.siso_ss1, u, t, x0)
         np.testing.assert_array_almost_equal(yout, youttrue, decimal=4)
 
-        #Test MIMO system, which contains ``siso_ss1`` twice
-        #first system: initial value, second system: step response
-        u = np.array([[0., 1.], [0, 1], [0, 1], [0, 1], [0, 1],
-                      [0, 1], [0, 1], [0, 1], [0, 1], [0, 1]])
-        x0 = np.matrix(".5; 1; 0; 0")
-        youttrue = np.array([[11., 9.], [8.1494, 17.6457], [5.9361, 24.7072],
-                             [4.2258, 30.4855], [2.9118, 35.2234],
-                             [1.9092, 39.1165], [1.1508, 42.3227],
-                             [0.5833, 44.9694], [0.1645, 47.1599],
-                             [-0.1391, 48.9776]])
-        yout, _t, _xout = lsim(self.mimo_ss1, u, t, x0)
-        np.testing.assert_array_almost_equal(yout, youttrue, decimal=4)
+        if slycot_check():
+            #Test MIMO system, which contains ``siso_ss1`` twice
+            #first system: initial value, second system: step response
+            u = np.array([[0., 1.], [0, 1], [0, 1], [0, 1], [0, 1],
+                          [0, 1], [0, 1], [0, 1], [0, 1], [0, 1]])
+            x0 = np.matrix(".5; 1; 0; 0")
+            youttrue = np.array([[11., 9.], [8.1494, 17.6457],
+                                 [5.9361, 24.7072], [4.2258, 30.4855],
+                                 [2.9118, 35.2234], [1.9092, 39.1165],
+                                 [1.1508, 42.3227], [0.5833, 44.9694],
+                                 [0.1645, 47.1599], [-0.1391, 48.9776]])
+            yout, _t, _xout = lsim(self.mimo_ss1, u, t, x0)
+            np.testing.assert_array_almost_equal(yout, youttrue, decimal=4)
 
     def testMargin(self):
         #! TODO: check results to make sure they are OK
@@ -238,7 +285,7 @@ class TestMatlab(unittest.TestCase):
         gm, pm, wg, wp = margin(self.siso_ss2);
         gm, pm, wg, wp = margin(self.siso_ss2*self.siso_ss2*2);
         np.testing.assert_array_almost_equal(
-            [gm, pm, wg, wp], [1.5451, 75.9933, 0.6559, 1.2720], decimal=3)
+            [gm, pm, wg, wp], [1.5451, 75.9933, 1.2720, 0.6559], decimal=3)
 
     def testDcgain(self):
         #Create different forms of a SISO system
@@ -253,26 +300,27 @@ class TestMatlab(unittest.TestCase):
         gain_zpk = dcgain(Z, P, k)
         gain_numden = dcgain(np.squeeze(num), den)
         gain_sys_ss = dcgain(sys_ss)
-        print('\ngain_abcd:', gain_abcd, 'gain_zpk:', gain_zpk)
-        print('gain_numden:', gain_numden, 'gain_sys_ss:', gain_sys_ss)
+        # print('\ngain_abcd:', gain_abcd, 'gain_zpk:', gain_zpk)
+        # print('gain_numden:', gain_numden, 'gain_sys_ss:', gain_sys_ss)
 
         #Compute the gain with a long simulation
         t = linspace(0, 1000, 1000)
         y, _t = step(sys_ss, t)
         gain_sim = y[-1]
-        print('gain_sim:', gain_sim)
+        # print('gain_sim:', gain_sim)
 
         #All gain values must be approximately equal to the known gain
         np.testing.assert_array_almost_equal(
-            [gain_abcd[0,0], gain_zpk[0,0], gain_numden[0,0], gain_sys_ss[0,0],
+            [gain_abcd, gain_zpk, gain_numden, gain_sys_ss,
              gain_sim],
             [59, 59, 59, 59, 59])
 
-        # Test with MIMO system, which contains ``siso_ss1`` twice
-        gain_mimo = dcgain(self.mimo_ss1)
-        print('gain_mimo: \n', gain_mimo)
-        np.testing.assert_array_almost_equal(gain_mimo, [[59., 0 ],
-                                                         [0,  59.]])
+        if slycot_check():
+            # Test with MIMO system, which contains ``siso_ss1`` twice
+            gain_mimo = dcgain(self.mimo_ss1)
+            # print('gain_mimo: \n', gain_mimo)
+            np.testing.assert_array_almost_equal(gain_mimo, [[59., 0 ],
+                                                             [0,  59.]])
 
     def testBode(self):
         bode(self.siso_ss1)
@@ -290,7 +338,10 @@ class TestMatlab(unittest.TestCase):
         rlocus(self.siso_ss1)
         rlocus(self.siso_tf1)
         rlocus(self.siso_tf2)
-        rlist, klist = rlocus(self.siso_tf2, klist=[1, 10, 100], Plot=False)
+        klist = [1, 10, 100]
+        rlist, klist_out = rlocus(self.siso_tf2, klist, Plot=False)
+        np.testing.assert_equal(len(rlist), len(klist))
+        np.testing.assert_array_equal(klist, klist_out)
 
     def testNyquist(self):
         nyquist(self.siso_ss1)
@@ -325,31 +376,52 @@ class TestMatlab(unittest.TestCase):
         evalfr(self.siso_tf1, w)
         evalfr(self.siso_tf2, w)
         evalfr(self.siso_tf3, w)
-        np.testing.assert_array_almost_equal(
-            evalfr(self.mimo_ss1, w),
-            np.array( [[44.8-21.4j, 0.], [0., 44.8-21.4j]]))
+        if slycot_check():
+            np.testing.assert_array_almost_equal(
+                evalfr(self.mimo_ss1, w),
+                np.array( [[44.8-21.4j, 0.], [0., 44.8-21.4j]]))
 
+    @unittest.skipIf(not slycot_check(), "slycot not installed")
     def testHsvd(self):
         hsvd(self.siso_ss1)
         hsvd(self.siso_ss2)
         hsvd(self.siso_ss3)
 
+    @unittest.skipIf(not slycot_check(), "slycot not installed")
     def testBalred(self):
         balred(self.siso_ss1, 1)
         balred(self.siso_ss2, 2)
         balred(self.siso_ss3, [2, 2])
 
+    @unittest.skipIf(not slycot_check(), "slycot not installed")
     def testModred(self):
         modred(self.siso_ss1, [1])
-        modred(self.siso_ss2 * self.siso_ss3, [1, 2])
-        modred(self.siso_ss3, [1], 'matchdc')
-        modred(self.siso_ss3, [1], 'truncate')
+        modred(self.siso_ss2 * self.siso_ss1, [0, 1])
+        modred(self.siso_ss1, [1], 'matchdc')
+        modred(self.siso_ss1, [1], 'truncate')
+
+    @unittest.skipIf(not slycot_check(), "slycot not installed")
+    def testPlace_varga(self):
+        place_varga(self.siso_ss1.A, self.siso_ss1.B, [-2, -2])
 
     def testPlace(self):
-        place(self.siso_ss1.A, self.siso_ss1.B, [-2, -2])
+        place(self.siso_ss1.A, self.siso_ss1.B, [-2, -2.5])
 
+    def testAcker(self):
+        acker(self.siso_ss1.A, self.siso_ss1.B, [-2, -2.5])
+
+
+    @unittest.skipIf(not slycot_check(), "slycot not installed")
     def testLQR(self):
         (K, S, E) = lqr(self.siso_ss1.A, self.siso_ss1.B, np.eye(2), np.eye(1))
+
+        # Should work if [Q N;N' R] is positive semi-definite
+        (K, S, E) = lqr(self.siso_ss2.A, self.siso_ss2.B, 10*np.eye(3), \
+                            np.eye(1), [[1], [1], [2]])
+
+    @unittest.skip("check not yet implemented")
+    def testLQR_checks(self):
+        # Make sure we get a warning if [Q N;N' R] is not positive semi-definite
         (K, S, E) = lqr(self.siso_ss2.A, self.siso_ss2.B, np.eye(3), \
                             np.eye(1), [[1], [1], [2]])
 
@@ -371,6 +443,7 @@ class TestMatlab(unittest.TestCase):
         obsv(self.siso_ss1.A, self.siso_ss1.C)
         obsv(self.siso_ss2.A, self.siso_ss2.C)
 
+    @unittest.skipIf(not slycot_check(), "slycot not installed")
     def testGram(self):
         gram(self.siso_ss1, 'c')
         gram(self.siso_ss2, 'c')
@@ -407,6 +480,7 @@ class TestMatlab(unittest.TestCase):
         for i in range(len(ssdata_1)):
             np.testing.assert_array_almost_equal(ssdata_1[i], ssdata_2[i])
 
+    @unittest.skipIf(not slycot_check(), "slycot not installed")
     def testMIMOssdata(self):
         m = (self.mimo_ss1.A, self.mimo_ss1.B, self.mimo_ss1.C, self.mimo_ss1.D)
         ssdata_1 = ssdata(self.mimo_ss1);
@@ -432,7 +506,7 @@ class TestMatlab(unittest.TestCase):
         D = np.zeros((4,2))
         sys = ss(A, B, C, D)
         wn, Z, p = damp(sys, False)
-        print (wn)
+        # print (wn)
         np.testing.assert_array_almost_equal(
             wn, np.array([4.07381994,   3.28874827,   3.28874827,
                           1.08937685e-03]))
@@ -445,7 +519,7 @@ class TestMatlab(unittest.TestCase):
         sys = append(sys1, sys2)
         Q= np.mat([ [ 1, 2], [2, -1] ]) # basically feedback, output 2 in 1
         sysc = connect(sys, Q, [2], [1, 2])
-        print(sysc)
+        # print(sysc)
         np.testing.assert_array_almost_equal(
             sysc.A, np.mat('1 -2 5; 3 -4 7; -6 -8 -10'))
         np.testing.assert_array_almost_equal(
@@ -487,6 +561,7 @@ class TestMatlab(unittest.TestCase):
         frd2 = frd(frd1.fresp[0,0,:], omega)
         assert isinstance(frd2, FRD)
 
+    @unittest.skipIf(not slycot_check(), "slycot not installed")
     def testMinreal(self, verbose=False):
         """Test a minreal model reduction"""
         #A = [-2, 0.5, 0; 0.5, -0.3, 0; 0, 0, -0.1]
@@ -500,7 +575,7 @@ class TestMatlab(unittest.TestCase):
         # sys = ss(A, B, C, D)
 
         sys = ss(A, B, C, D)
-        sysr = minreal(sys)
+        sysr = minreal(sys, verbose=verbose)
         self.assertEqual(sysr.states, 2)
         self.assertEqual(sysr.inputs, sys.inputs)
         self.assertEqual(sysr.outputs, sys.outputs)
@@ -509,12 +584,11 @@ class TestMatlab(unittest.TestCase):
 
         s = tf([1, 0], [1])
         h = (s+1)*(s+2.00000000001)/(s+2)/(s**2+s+1)
-        hm = minreal(h)
+        hm = minreal(h, verbose=verbose)
         hr = (s+1)/(s**2+s+1)
         np.testing.assert_array_almost_equal(hm.num[0][0], hr.num[0][0])
         np.testing.assert_array_almost_equal(hm.den[0][0], hr.den[0][0])
 
-    @unittest.skip("skipping testSS2cont: not implemented for MIMO")
     def testSS2cont(self):
         sys = ss(
             np.mat("-3 4 2; -1 -3 0; 2 5 3"),
@@ -533,8 +607,6 @@ class TestMatlab(unittest.TestCase):
                       -0.260952977031384  -0.274201791021713;
                       -0.304617775734327   0.075182622718853"""), sysd.B)
 
-
-    @unittest.skip("skipping testCombi01: need to check/update margins")
     def testCombi01(self):
         # test from a "real" case, combines tf, ss, connect and margin
         # this is a type 2 system, with phase starting at -180. The
@@ -586,10 +658,11 @@ class TestMatlab(unittest.TestCase):
         Hol = Hc*Hno*Hp
 
         gm, pm, wg, wp = margin(Hol)
+        # print("%f %f %f %f" % (gm, pm, wg, wp))
         self.assertAlmostEqual(gm, 3.32065569155)
         self.assertAlmostEqual(pm, 46.9740430224)
-        self.assertAlmostEqual(wp, 0.0616288455466)
         self.assertAlmostEqual(wg, 0.176469728448)
+        self.assertAlmostEqual(wp, 0.0616288455466)
 
 #! TODO: not yet implemented
 #    def testMIMOtfdata(self):
@@ -599,7 +672,7 @@ class TestMatlab(unittest.TestCase):
 #        for i in range(len(tfdata)):
 #            np.testing.assert_array_almost_equal(tfdata_1[i], tfdata_2[i])
 
-def suite():
+def test_suite():
    return unittest.TestLoader().loadTestsFromTestCase(TestMatlab)
 
 if __name__ == '__main__':
