@@ -5,55 +5,57 @@
 
 import unittest
 import numpy as np
-from control.statesp import StateSpace, _convertToStateSpace
-from control.xferfcn import TransferFunction, _convertToTransferFunction
-from control.lti import isdtime
+from control.statesp import StateSpace, _convertToStateSpace, rss
+from control.xferfcn import TransferFunction, _convertToTransferFunction, ss2tf
+from control.lti import evalfr
+from control.exception import slycot_check
+# from control.lti import isdtime
 
 class TestXferFcn(unittest.TestCase):
     """These are tests for functionality and correct reporting of the transfer
     function class.  Throughout these tests, we will give different input
     formats to the xTranferFunction constructor, to try to break it.  These
     tests have been verified in MATLAB."""
-    
+
     # Tests for raising exceptions.
-   
+
     def testBadInputType(self):
         """Give the constructor invalid input types."""
-        
+
         self.assertRaises(TypeError, TransferFunction, [[0., 1.], [2., 3.]],
             [[5., 2.], [3., 0.]])
-            
+
     def testInconsistentDimension(self):
         """Give the constructor a numerator and denominator of different
         sizes."""
-        
+
         self.assertRaises(ValueError, TransferFunction, [[[1.]]],
             [[[1.], [2., 3.]]])
         self.assertRaises(ValueError, TransferFunction, [[[1.]]],
             [[[1.]], [[2., 3.]]])
         self.assertRaises(ValueError, TransferFunction, [[[1.]]],
             [[[1.], [1., 2.]], [[5., 2.], [2., 3.]]])
-    
+
     def testInconsistentColumns(self):
         """Give the constructor inputs that do not have the same number of
         columns in each row."""
-        
+
         self.assertRaises(ValueError, TransferFunction, 1.,
             [[[1.]], [[2.], [3.]]])
         self.assertRaises(ValueError, TransferFunction, [[[1.]], [[2.], [3.]]],
             1.)
-            
+
     def testZeroDenominator(self):
         """Give the constructor a transfer function with a zero denominator."""
-        
+
         self.assertRaises(ValueError, TransferFunction, 1., 0.)
         self.assertRaises(ValueError, TransferFunction,
             [[[1.], [2., 3.]], [[-1., 4.], [3., 2.]]],
             [[[1., 0.], [0.]], [[0., 0.], [2.]]])
-            
+
     def testAddInconsistentDimension(self):
         """Add two transfer function matrices of different sizes."""
-        
+
         sys1 = TransferFunction([[[1., 2.]]], [[[4., 5.]]])
         sys2 = TransferFunction([[[4., 3.]], [[1., 2.]]],
             [[[1., 6.]], [[2., 4.]]])
@@ -61,102 +63,105 @@ class TestXferFcn(unittest.TestCase):
         self.assertRaises(ValueError, sys1.__sub__, sys2)
         self.assertRaises(ValueError, sys1.__radd__, sys2)
         self.assertRaises(ValueError, sys1.__rsub__, sys2)
-        
+
     def testMulInconsistentDimension(self):
         """Multiply two transfer function matrices of incompatible sizes."""
-        
+
         sys1 = TransferFunction([[[1., 2.], [4., 5.]], [[2., 5.], [4., 3.]]],
             [[[6., 2.], [4., 1.]], [[6., 7.], [2., 4.]]])
-        sys2 = TransferFunction([[[1.]], [[2.]], [[3.]]], 
+        sys2 = TransferFunction([[[1.]], [[2.]], [[3.]]],
             [[[4.]], [[5.]], [[6.]]])
         self.assertRaises(ValueError, sys1.__mul__, sys2)
         self.assertRaises(ValueError, sys2.__mul__, sys1)
         self.assertRaises(ValueError, sys1.__rmul__, sys2)
         self.assertRaises(ValueError, sys2.__rmul__, sys1)
-    
+
     # Tests for TransferFunction._truncatecoeff
-    
+
     def testTruncateCoeff1(self):
         """Remove extraneous zeros in polynomial representations."""
-        
+
         sys1 = TransferFunction([0., 0., 1., 2.], [[[0., 0., 0., 3., 2., 1.]]])
-        
+
         np.testing.assert_array_equal(sys1.num, [[[1., 2.]]])
         np.testing.assert_array_equal(sys1.den, [[[3., 2., 1.]]])
-        
+
     def testTruncateCoeff2(self):
         """Remove extraneous zeros in polynomial representations."""
-        
+
         sys1 = TransferFunction([0., 0., 0.], 1.)
-        
+
         np.testing.assert_array_equal(sys1.num, [[[0.]]])
         np.testing.assert_array_equal(sys1.den, [[[1.]]])
-    
+
     # Tests for TransferFunction.__neg__
-    
+
     def testNegScalar(self):
         """Negate a direct feedthrough system."""
-        
-        sys1 = TransferFunction(2., np.array([-3]))
+
+        sys1 = TransferFunction(2., np.array([-3.]))
         sys2 = - sys1
-        
+
         np.testing.assert_array_equal(sys2.num, [[[-2.]]])
         np.testing.assert_array_equal(sys2.den, [[[-3.]]])
-    
+
     def testNegSISO(self):
         """Negate a SISO system."""
-        
+
         sys1 = TransferFunction([1., 3., 5], [1., 6., 2., -1.])
         sys2 = - sys1
-        
+
         np.testing.assert_array_equal(sys2.num, [[[-1., -3., -5.]]])
         np.testing.assert_array_equal(sys2.den, [[[1., 6., 2., -1.]]])
-        
+
+
+    @unittest.skipIf(not slycot_check(), "slycot not installed")
     def testNegMIMO(self):
         """Negate a MIMO system."""
-    
+
         num1 = [[[1., 2.], [0., 3.], [2., -1.]],
                 [[1.], [4., 0.], [1., -4., 3.]]]
         num3 = [[[-1., -2.], [0., -3.], [-2., 1.]],
                 [[-1.], [-4., 0.], [-1., 4., -3.]]]
         den1 = [[[-3., 2., 4.], [1., 0., 0.], [2., -1.]],
                 [[3., 0., .0], [2., -1., -1.], [1.]]]
-                
+
         sys1 = TransferFunction(num1, den1)
         sys2 = - sys1
         sys3 = TransferFunction(num3, den1)
-        
+
         for i in range(sys3.outputs):
             for j in range(sys3.inputs):
                 np.testing.assert_array_equal(sys2.num[i][j], sys3.num[i][j])
                 np.testing.assert_array_equal(sys2.den[i][j], sys3.den[i][j])
-               
+
     # Tests for TransferFunction.__add__
-    
+
     def testAddScalar(self):
         """Add two direct feedthrough systems."""
-        
+
         sys1 = TransferFunction(1., [[[1.]]])
         sys2 = TransferFunction(np.array([2.]), [1.])
         sys3 = sys1 + sys2
-        
+
         np.testing.assert_array_equal(sys3.num, 3.)
         np.testing.assert_array_equal(sys3.den, 1.)
 
     def testAddSISO(self):
         """Add two SISO systems."""
-        
+
         sys1 = TransferFunction([1., 3., 5], [1., 6., 2., -1])
         sys2 = TransferFunction([[np.array([-1., 3.])]], [[[1., 0., -1.]]])
         sys3 = sys1 + sys2
-        
+
         # If sys3.num is [[[0., 20., 4., -8.]]], then this is wrong!
         np.testing.assert_array_equal(sys3.num, [[[20., 4., -8]]])
         np.testing.assert_array_equal(sys3.den, [[[1., 6., 1., -7., -2., 1.]]])
-        
+
+    @unittest.skipIf(not slycot_check(), "slycot not installed")
     def testAddMIMO(self):
         """Add two MIMO systems."""
-        
+
         num1 = [[[1., 2.], [0., 3.], [2., -1.]],
                 [[1.], [4., 0.], [1., -4., 3.]]]
         den1 = [[[-3., 2., 4.], [1., 0., 0.], [2., -1.]],
@@ -169,7 +174,7 @@ class TestXferFcn(unittest.TestCase):
                 [[3., 2., -3., 2], [-2., -3., 7., 2.], [1., -4., 3., 4]]]
         den3 = [[[3., -2., -4.], [1., 2., 3., 0., 0.], [-2., -1., 1.]],
                 [[-12., -9., 6., 0., 0.], [2., -1., -1.], [1., 0.]]]
-                
+
         sys1 = TransferFunction(num1, den1)
         sys2 = TransferFunction(num2, den2)
         sys3 = sys1 + sys2
@@ -178,35 +183,36 @@ class TestXferFcn(unittest.TestCase):
             for j in range(sys3.inputs):
                 np.testing.assert_array_equal(sys3.num[i][j], num3[i][j])
                 np.testing.assert_array_equal(sys3.den[i][j], den3[i][j])
-    
+
     # Tests for TransferFunction.__sub__
-    
+
     def testSubScalar(self):
         """Add two direct feedthrough systems."""
-        
+
         sys1 = TransferFunction(1., [[[1.]]])
         sys2 = TransferFunction(np.array([2.]), [1.])
         sys3 = sys1 - sys2
-        
+
         np.testing.assert_array_equal(sys3.num, -1.)
         np.testing.assert_array_equal(sys3.den, 1.)
 
     def testSubSISO(self):
         """Add two SISO systems."""
-        
+
         sys1 = TransferFunction([1., 3., 5], [1., 6., 2., -1])
         sys2 = TransferFunction([[np.array([-1., 3.])]], [[[1., 0., -1.]]])
         sys3 = sys1 - sys2
         sys4 = sys2 - sys1
-        
+
         np.testing.assert_array_equal(sys3.num, [[[2., 6., -12., -10., -2.]]])
         np.testing.assert_array_equal(sys3.den, [[[1., 6., 1., -7., -2., 1.]]])
         np.testing.assert_array_equal(sys4.num, [[[-2., -6., 12., 10., 2.]]])
         np.testing.assert_array_equal(sys4.den, [[[1., 6., 1., -7., -2., 1.]]])
-        
+
+    @unittest.skipIf(not slycot_check(), "slycot not installed")
     def testSubMIMO(self):
         """Add two MIMO systems."""
-        
+
         num1 = [[[1., 2.], [0., 3.], [2., -1.]],
                 [[1.], [4., 0.], [1., -4., 3.]]]
         den1 = [[[-3., 2., 4.], [1., 0., 0.], [2., -1.]],
@@ -219,7 +225,7 @@ class TestXferFcn(unittest.TestCase):
                 [[-3., -10., -3., 2], [2., 3., 1., -2], [1., -4., 3., -4]]]
         den3 = [[[3., -2., -4], [1., 2., 3., 0., 0.], [1]],
                 [[-12., -9., 6., 0., 0.], [2., -1., -1], [1., 0.]]]
-                
+
         sys1 = TransferFunction(num1, den1)
         sys2 = TransferFunction(num2, den2)
         sys3 = sys1 - sys2
@@ -228,38 +234,39 @@ class TestXferFcn(unittest.TestCase):
             for j in range(sys3.inputs):
                 np.testing.assert_array_equal(sys3.num[i][j], num3[i][j])
                 np.testing.assert_array_equal(sys3.den[i][j], den3[i][j])
-               
+
     # Tests for TransferFunction.__mul__
-    
+
     def testMulScalar(self):
         """Multiply two direct feedthrough systems."""
-        
+
         sys1 = TransferFunction(2., [1.])
         sys2 = TransferFunction(1., 4.)
         sys3 = sys1 * sys2
         sys4 = sys1 * sys2
-        
+
         np.testing.assert_array_equal(sys3.num, [[[2.]]])
         np.testing.assert_array_equal(sys3.den, [[[4.]]])
         np.testing.assert_array_equal(sys3.num, sys4.num)
         np.testing.assert_array_equal(sys3.den, sys4.den)
-        
+
     def testMulSISO(self):
         """Multiply two SISO systems."""
-        
+
         sys1 = TransferFunction([1., 3., 5], [1., 6., 2., -1])
         sys2 = TransferFunction([[[-1., 3.]]], [[[1., 0., -1.]]])
         sys3 = sys1 * sys2
         sys4 = sys2 * sys1
-        
+
         np.testing.assert_array_equal(sys3.num, [[[-1., 0., 4., 15.]]])
         np.testing.assert_array_equal(sys3.den, [[[1., 6., 1., -7., -2., 1.]]])
         np.testing.assert_array_equal(sys3.num, sys4.num)
         np.testing.assert_array_equal(sys3.den, sys4.den)
-        
+
+    @unittest.skipIf(not slycot_check(), "slycot not installed")
     def testMulMIMO(self):
         """Multiply two MIMO systems."""
-        
+
         num1 = [[[1., 2.], [0., 3.], [2., -1.]],
                 [[1.], [4., 0.], [1., -4., 3.]]]
         den1 = [[[-3., 2., 4.], [1., 0., 0.], [2., -1.]],
@@ -277,58 +284,72 @@ class TestXferFcn(unittest.TestCase):
         den3 = [[[48., -92., -84., 183., 44., -97., -2., 12., 0., 0., 0., 0.,
                   0., 0.]],
                 [[-48., 60., 84., -81., -45., 21., 9., 0., 0., 0., 0., 0., 0.]]]
-        
+
         sys1 = TransferFunction(num1, den1)
         sys2 = TransferFunction(num2, den2)
         sys3 = sys1 * sys2
-        
+
         for i in range(sys3.outputs):
             for j in range(sys3.inputs):
                 np.testing.assert_array_equal(sys3.num[i][j], num3[i][j])
                 np.testing.assert_array_equal(sys3.den[i][j], den3[i][j])
 
     # Tests for TransferFunction.__div__
-    
+
     def testDivScalar(self):
         """Divide two direct feedthrough systems."""
-        
+
         sys1 = TransferFunction(np.array([3.]), -4.)
         sys2 = TransferFunction(5., 2.)
         sys3 = sys1 / sys2
-        
+
         np.testing.assert_array_equal(sys3.num, [[[6.]]])
         np.testing.assert_array_equal(sys3.den, [[[-20.]]])
-        
+
     def testDivSISO(self):
         """Divide two SISO systems."""
-        
+
         sys1 = TransferFunction([1., 3., 5], [1., 6., 2., -1])
         sys2 = TransferFunction([[[-1., 3.]]], [[[1., 0., -1.]]])
         sys3 = sys1 / sys2
         sys4 = sys2 / sys1
-        
+
         np.testing.assert_array_equal(sys3.num, [[[1., 3., 4., -3., -5.]]])
         np.testing.assert_array_equal(sys3.den, [[[-1., -3., 16., 7., -3.]]])
         np.testing.assert_array_equal(sys4.num, sys3.den)
         np.testing.assert_array_equal(sys4.den, sys3.num)
-        
-    # Tests for TransferFunction.evalfr.
 
+    # Tests for TransferFunction.evalfr.
     def testEvalFrSISO(self):
         """Evaluate the frequency response of a SISO system at one frequency."""
 
         sys = TransferFunction([1., 3., 5], [1., 6., 2., -1])
 
-        np.testing.assert_array_almost_equal(sys.evalfr(1.),
+        np.testing.assert_array_almost_equal(evalfr(sys, 1j),
             np.array([[-0.5 - 0.5j]]))
-        np.testing.assert_array_almost_equal(sys.evalfr(32.),
+        np.testing.assert_array_almost_equal(evalfr(sys, 32j),
             np.array([[0.00281959302585077 - 0.030628473607392j]]))
 
         # Test call version as well
         np.testing.assert_almost_equal(sys(1.j), -0.5 - 0.5j)
-        np.testing.assert_almost_equal(sys(32.j), 
+        np.testing.assert_almost_equal(sys(32.j),
             0.00281959302585077 - 0.030628473607392j)
 
+        # Test internal version (with real argument)
+        np.testing.assert_array_almost_equal(sys._evalfr(1.),
+            np.array([[-0.5 - 0.5j]]))
+        np.testing.assert_array_almost_equal(sys._evalfr(32.),
+            np.array([[0.00281959302585077 - 0.030628473607392j]]))
+
+        # Deprecated version of the call (should generate warning)
+        import warnings
+        with warnings.catch_warnings(record=True) as w:
+            warnings.simplefilter("always")
+            sys.evalfr(1.)
+            assert len(w) == 1
+            assert issubclass(w[-1].category, PendingDeprecationWarning)
+
+    @unittest.skipIf(not slycot_check(), "slycot not installed")
     def testEvalFrMIMO(self):
         """Evaluate the frequency response of a MIMO system at one frequency."""
 
@@ -340,8 +361,8 @@ class TestXferFcn(unittest.TestCase):
         resp = [[0.147058823529412 + 0.0882352941176471j, -0.75, 1.],
                 [-0.083333333333333, -0.188235294117647 - 0.847058823529412j,
                  -1. - 8.j]]
-        
-        np.testing.assert_array_almost_equal(sys.evalfr(2.), resp)
+
+        np.testing.assert_array_almost_equal(sys._evalfr(2.), resp)
 
         # Test call version as well
         np.testing.assert_array_almost_equal(sys(2.j), resp)
@@ -365,6 +386,7 @@ class TestXferFcn(unittest.TestCase):
         np.testing.assert_array_almost_equal(phase, truephase)
         np.testing.assert_array_almost_equal(omega, trueomega)
 
+    @unittest.skipIf(not slycot_check(), "slycot not installed")
     def testFreqRespMIMO(self):
         """Evaluate the magnitude and phase of a MIMO system at multiple
         frequencies."""
@@ -374,7 +396,7 @@ class TestXferFcn(unittest.TestCase):
         den = [[[-3., 2., 4.], [1., 0., 0.], [2., -1.]],
                [[3., 0., .0], [2., -1., -1.], [1.]]]
         sys = TransferFunction(num, den)
-        
+
         trueomega = [0.1, 1., 10.]
         truemag = [[[0.496287094505259, 0.307147558416976, 0.0334738176210382],
                     [300., 3., 0.03], [1., 1., 1.]],
@@ -394,7 +416,8 @@ class TestXferFcn(unittest.TestCase):
         np.testing.assert_array_equal(omega, trueomega)
 
     # Tests for TransferFunction.pole and TransferFunction.zero.
-
+    
+    @unittest.skipIf(not slycot_check(), "slycot not installed")
     def testPoleMIMO(self):
         """Test for correct MIMO poles."""
 
@@ -402,10 +425,16 @@ class TestXferFcn(unittest.TestCase):
             [[[1., 2.], [1., 3.]], [[1., 4., 4.], [1., 9., 14.]]])
         p = sys.pole()
 
-        np.testing.assert_array_almost_equal(p, [-7., -3., -2., -2.])
+        np.testing.assert_array_almost_equal(p, [-2., -2., -7., -3., -2.])
 
-    # Tests for TransferFunction.feedback.
+    @unittest.skipIf(not slycot_check(), "slycot not installed")
+    def testDoubleCancelingPoleSiso(self):
         
+        H = TransferFunction([1,1],[1,2,1])
+        p = H.pole()
+        np.testing.assert_array_almost_equal(p, [-1, -1])
+    
+    # Tests for TransferFunction.feedback
     def testFeedbackSISO(self):
         """Test for correct SISO transfer function feedback."""
 
@@ -419,7 +448,8 @@ class TestXferFcn(unittest.TestCase):
         np.testing.assert_array_equal(sys3.den, [[[1., 0., -2., 2., 32., 0.]]])
         np.testing.assert_array_equal(sys4.num, [[[-1., 7., -16., 16., 0.]]])
         np.testing.assert_array_equal(sys4.den, [[[1., 0., 2., -8., 8., 0.]]])
-             
+
+    @unittest.skipIf(not slycot_check(), "slycot not installed")
     def testConvertToTransferFunction(self):
         """Test for correct state space to transfer function conversion."""
 
@@ -434,7 +464,7 @@ class TestXferFcn(unittest.TestCase):
         num = [[np.array([1., -7., 10.]), np.array([-1., 10.])],
                [np.array([2., -8.]), np.array([1., -2., -8.])],
                [np.array([1., 1., -30.]), np.array([7., -22.])]]
-        den = [[np.array([1., -5., -2.]) for j in range(sys.inputs)] 
+        den = [[np.array([1., -5., -2.]) for j in range(sys.inputs)]
             for i in range(sys.outputs)]
 
         for i in range(sys.outputs):
@@ -451,9 +481,10 @@ class TestXferFcn(unittest.TestCase):
         hr = (s+1)/(s**2+s+1)
         np.testing.assert_array_almost_equal(hm.num[0][0], hr.num[0][0])
         np.testing.assert_array_almost_equal(hm.den[0][0], hr.den[0][0])
+        np.testing.assert_equal(hm.dt, hr.dt)
 
     def testMinreal2(self):
-        """This one gave a problem, due to poly([]) giving simply 1 
+        """This one gave a problem, due to poly([]) giving simply 1
         instead of numpy.array([1])"""
         s = TransferFunction([1, 0], [1])
         G = 6205/(s*(s**2 + 13*s + 1281))
@@ -464,7 +495,26 @@ class TestXferFcn(unittest.TestCase):
         hr = 6205/(s**2+8*s+1241)
         np.testing.assert_array_almost_equal(H2b.num[0][0], hr.num[0][0])
         np.testing.assert_array_almost_equal(H2b.den[0][0], hr.den[0][0])
+        np.testing.assert_equal(H2b.dt, hr.dt)
 
+    def testMinreal3(self):
+        """Regression test for minreal of tf([1,1],[1,1])"""
+        g = TransferFunction([1,1],[1,1]).minreal()
+        np.testing.assert_array_almost_equal(1.0, g.num[0][0])
+        np.testing.assert_array_almost_equal(1.0, g.den[0][0])
+        np.testing.assert_equal(None, g.dt)
+
+    def testMinreal4(self):
+        """Check minreal on discrete TFs."""
+        T = 0.01
+        z = TransferFunction([1, 0], [1], T)
+        h = (z-1.00000000001)*(z+1.0000000001)/((z**2-1))
+        hm = h.minreal()
+        hr = TransferFunction([1], [1], T)
+        np.testing.assert_array_almost_equal(hm.num[0][0], hr.num[0][0])
+        np.testing.assert_equal(hr.dt, hm.dt)
+
+    @unittest.skipIf(not slycot_check(), "slycot not installed")
     def testMIMO(self):
         """Test conversion of a single input, two-output state-space
         system against the same TF"""
@@ -477,7 +527,7 @@ class TestXferFcn(unittest.TestCase):
         a2 = 3.6
         a3 = 1.0
         h = (b0 + b1*s + b2*s**2)/(a0 + a1*s + a2*s**2 + a3*s**3)
-        H = TransferFunction([[h.num[0][0]], [(h*s).num[0][0]]], 
+        H = TransferFunction([[h.num[0][0]], [(h*s).num[0][0]]],
                              [[h.den[0][0]], [h.den[0][0]]])
         sys = _convertToStateSpace(H)
         H2 = _convertToTransferFunction(sys)
@@ -486,8 +536,28 @@ class TestXferFcn(unittest.TestCase):
         np.testing.assert_array_almost_equal(H.num[1][0], H2.num[1][0])
         np.testing.assert_array_almost_equal(H.den[1][0], H2.den[1][0])
 
+    @unittest.skipIf(not slycot_check(), "slycot not installed")
+    def testIndexing(self):
+        tm = ss2tf(rss(5, 3, 3))
+
+        # scalar indexing
+        sys01 = tm[0, 1]
+        np.testing.assert_array_almost_equal(sys01.num[0][0], tm.num[0][1])
+        np.testing.assert_array_almost_equal(sys01.den[0][0], tm.den[0][1])
+
+        # slice indexing
+        sys = tm[:2, 1:3]
+        np.testing.assert_array_almost_equal(sys.num[0][0], tm.num[0][1])
+        np.testing.assert_array_almost_equal(sys.den[0][0], tm.den[0][1])
+        np.testing.assert_array_almost_equal(sys.num[0][1], tm.num[0][2])
+        np.testing.assert_array_almost_equal(sys.den[0][1], tm.den[0][2])
+        np.testing.assert_array_almost_equal(sys.num[1][0], tm.num[1][1])
+        np.testing.assert_array_almost_equal(sys.den[1][0], tm.den[1][1])
+        np.testing.assert_array_almost_equal(sys.num[1][1], tm.num[1][2])
+        np.testing.assert_array_almost_equal(sys.den[1][1], tm.den[1][2])
+
     def testMatrixMult(self):
-        """MIMO transfer functions should be multiplyable by constant 
+        """MIMO transfer functions should be multiplyable by constant
         matrices"""
         s = TransferFunction([1, 0], [1])
         b0 = 0.2
@@ -498,7 +568,7 @@ class TestXferFcn(unittest.TestCase):
         a2 = 3.6
         a3 = 1.0
         h = (b0 + b1*s + b2*s**2)/(a0 + a1*s + a2*s**2 + a3*s**3)
-        H = TransferFunction([[h.num[0][0]], [(h*s).num[0][0]]], 
+        H = TransferFunction([[h.num[0][0]], [(h*s).num[0][0]]],
                              [[h.den[0][0]], [h.den[0][0]]])
         H1 = (np.matrix([[1.0, 0]])*H).minreal()
         H2 = (np.matrix([[0, 1.0]])*H).minreal()
@@ -507,6 +577,43 @@ class TestXferFcn(unittest.TestCase):
         np.testing.assert_array_almost_equal(H.num[1][0], H2.num[0][0])
         np.testing.assert_array_almost_equal(H.den[1][0], H2.den[0][0])
 
+    def test_dcgain_cont(self):
+        """Test DC gain for continuous-time transfer functions"""
+        sys = TransferFunction(6, 3)
+        np.testing.assert_equal(sys.dcgain(), 2)
+
+        sys2 = TransferFunction(6, [1, 3])
+        np.testing.assert_equal(sys2.dcgain(), 2)
+
+        sys3 = TransferFunction(6, [1, 0])
+        np.testing.assert_equal(sys3.dcgain(), np.inf)
+
+        num = [[[15], [21], [33]], [[10], [14], [22]]]
+        den = [[[1, 3], [2, 3], [3, 3]], [[1, 5], [2, 7], [3, 11]]]
+        sys4 = TransferFunction(num, den)
+        expected = [[5, 7, 11], [2, 2, 2]]
+        np.testing.assert_array_equal(sys4.dcgain(), expected)
+
+    def test_dcgain_discr(self):
+        """Test DC gain for discrete-time transfer functions"""
+        # static gain
+        sys = TransferFunction(6, 3, True)
+        np.testing.assert_equal(sys.dcgain(), 2)
+
+        # averaging filter
+        sys = TransferFunction(0.5, [1, -0.5], True)
+        np.testing.assert_almost_equal(sys.dcgain(), 1)
+
+        # differencer
+        sys = TransferFunction(1, [1, -1], True)
+        np.testing.assert_equal(sys.dcgain(), np.inf)
+
+        # summer
+        # causes a RuntimeWarning due to the divide by zero
+        sys = TransferFunction([1,-1], [1], True)
+        np.testing.assert_equal(sys.dcgain(), 0)
+
+
 def suite():
     return unittest.TestLoader().loadTestsFromTestCase(TestXferFcn)
 
